# Snohomish diagnostic plots from MARSS models
require(MARSS)
library(tidyverse)

# The peak flow data will be summarized for the incubation period for each population.
incubation_range <- list(
  c(pop = 'coho',           start = "1015", end = "0515"),
  c(pop = 'summer_chinook', start = "0910", end = "0415"),
  c(pop = 'fall_chinook',   start = "0920", end = "0510"),
  c(pop = 'winter_steelhead', start = "0301", end = "0910"),
  c(pop= 'summer_steelhead', start = "0301", end = "0910"))

# Daily discharge at Snohomish near Monroe
require(EGRET)
site_number_sno <- 12150800
start_date <- "1963-02-01"  # Get earliest date
end_date <- "2020-12-31"
daily_sno <- readNWISDaily(site_number_sno,"00060",start_date,end_date)
plot(daily_sno$Date, daily_sno$Q, type="l")# Q = m^3/s

par(mfrow=c(2,1))
plot(daily_sno$Date, daily_sno$Q * 35.31467, type="l")# Q = f^3/s
plot(daily_sno$Date, daily_sno$Q, type="l", col="red")# Q = m^3/s

# Find the annual maximum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
sno_q_peak <- map_dfr(incubation_range, function(r){
  daily_sno %>%
    mutate(monthday = format(Date, "%m%d"),
           pop = r['pop']) %>%
    filter(monthday <= r['start'] | monthday >= r['end']) %>%
    group_by(pop, waterYear) %>%
    summarize(Q_max = max(Q) * 35.31467)
})
sno_q <- pivot_wider(sno_q_peak,
                     names_from = pop,
                     names_prefix = 'peak_q_',
                     values_from = Q_max)
# Find the annual minimum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
# Calculating 7DADMin for low flow
sno_low_q7 <- tapply(daily_sno$Q7[daily_sno$waterYear>1962],
                     daily_sno$waterYear[daily_sno$waterYear>1962],
                     function(x, n = 7) {
                       min(as.vector(stats::filter(x, rep(1/n, n),sides=2)),
                           na.rm = TRUE)
                     })
sno_low_q7 <- data.frame(waterYear = names(sno_low_q7),
                         low_q7 = sno_low_q7 * 35.31467)

sno_q <- merge(sno_q, sno_low_q7)

# Stream temperature data from NorWest lower Snoqualmie
# Used to create relationship between max air temp and max water temp
snoq_stream_temp<-read.csv("Snoq_lower_stream_temp.csv")%>%
  mutate(start_date = as.Date(start_date),
         end_date = as.Date(end_date))

# Air temperature data
# USC00455525 is the ID for the Snohomish Monroe NOAA NCDC station
require(rnoaa)
require(purrr)
require(tidyverse)

t_max_yr <- map(c("USC00455525"), ghcnd_search,
             var = 'TMAX',
             date_min = '1920-01-01',
             date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y")))%>% #year only format for year_1 column
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)

t_max <- t_max_yr[35:93,] #1963 - 2021

# Subset air temperature time period to match stream temperature time series
t_max_regression <- map(c("USC00455525"), ghcnd_search,
                        var = 'TMAX',
                        date_min = '1920-01-01',
                        date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y"))) #year only format for year_1 column

date_subset <- function(start_date, end_date){
  filter(t_max_regression, date>=start_date, date<=end_date)

}

t_max_subset <- map2_dfr(snoq_stream_temp$start_date, snoq_stream_temp$end_date, date_subset)%>%
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)#NOAA data is downloaded as max temperature in tenths of degrees C

# We fit a relationship between Snohomish TMAX air temperature and NorWest  Snoqualmie 7DADM water temperature
# This relationship is used later to convert the MARSS simulation of air TMAX to water 7DADM
# Subset air temperature time period to match stream temperature time series
stream_air_temp <- merge(x=snoq_stream_temp,
                        y=t_max_subset, all.x = TRUE)
# Regression
lm_7dadm_tmax <- lm(T7dadm ~ tmaxC, stream_air_temp)
summary(lm_7dadm_tmax)
# Plot regression
plot(stream_air_temp$tmaxC, stream_air_temp$T7dadm, pch=21, bg="darkgrey", cex=3,
     xlab="Monroe Max Air (degC)", ylab="Lower Snoqualmie T7DADM Water", las=1,
     main= "Linear relationship between Monroe max air and Stillaguamish water 7DADM")
legend(x="topleft", ncol=1, legend=c("y = 8.0 + 0.42x", expression('r'^"2"*' = 0.10')),
       pch=c(20, NA), bty="n", text.width=1.3, col="white", pt.bg=c("white"), cex=1,
       pt.cex=2, lty=c(0,0))
abline(lm_7dadm_tmax)


# Model fits includes the data (black points), predictions (horizontal bars),
# confidence (red line segments) and prediction (gray line segments) intervals
plot(stream_air_temp$year, stream_air_temp$T7dadm,
     type="n", ylim=c(15,30), las = 1, axes = FALSE,
     xlab="Year", ylab = "7DADM (degC)")
axis(1, at=2012:2021, labels = 2012:2021)
# text(x = 1954:1974, xpd=NA,
#      y = 12, srt = 45, 1954:1974)
axis(2, las = 1)
#fitted
segments(x0 = stream_air_temp$year-0.3,
         x1 = stream_air_temp$year+0.3,
         y0 = predict(lm_7dadm_tmax),
         y1 = predict(lm_7dadm_tmax),
         lwd = 2, col = "black")
#prediction intervals
segments(x0 = stream_air_temp$year,
         x1 = stream_air_temp$year,
         y0 = predict(lm_7dadm_tmax, newdata=as.data.frame(stream_air_temp), interval="p")[,2],
         y1 = predict(lm_7dadm_tmax, newdata=as.data.frame(stream_air_temp), interval="p")[,3],
         lwd = 4, col = "gray70")
#confidence intervals
segments(x0 = stream_air_temp$year,
         x1 = stream_air_temp$year,
         y0 = predict(lm_7dadm_tmax, newdata=as.data.frame(stream_air_temp), interval="c")[,2],
         y1 = predict(lm_7dadm_tmax, newdata=as.data.frame(stream_air_temp), interval="c")[,3],
         lwd = 5, col ="orangered")
# observations
points(stream_air_temp$year, stream_air_temp$T7dadm,
       pch=21, bg="black")

# Series of plots of raw data
# Low flow
plot(sno_q$waterYear, sno_q$low_q7, type="l", las=1,
     ylab = "Snohomish low Q (cfs)", xlab = "Water Year")
# Peak flow
plot(sno_q$waterYear, sno_q$peak_q_coho, type="l", las=1,
     ylab = "Snohomish peak Q (cfs)", xlab = "Water Year")
lines(sno_q$waterYear, sno_q$peak_q_summer_chinook, col="blue")
lines(sno_q$waterYear, sno_q$peak_q_fall_chinook, col="orange")
lines(sno_q$waterYear, sno_q$peak_q_winter_steelhead, col="forestgreen")
lines(sno_q$waterYear, sno_q$peak_q_summer_steelhead, col="brown")
# Coho, fall Chinook, and summer Chinook are the same
# Summer and winter steelhead are the same

# 7DADM water temperature
plot(snoq_stream_temp$year, snoq_stream_temp$T7dadm, type = "l",
     ylab="7DADM (GM, degC)",
     xlim=c(min(sno_q$waterYear), max(sno_q$waterYear)), las=1)

# Max air temperature
plot(t_max$year, t_max$tmaxC, type="l", las=1,
     xlab="Year", ylab="Air TMAX (C)", main = "Monroe stationid USC00455525")

# Distribution plots
# Max air temperature
par(mfrow=c(2,2))
plot(density(t_max$tmaxC))
hist(t_max$tmaxC)
qqnorm(t_max$tmaxC)
qqline(t_max$tmaxC)
# Log transformation
par(mfrow=c(2,2))
plot(density(log(t_max$tmaxC)))
hist(log(t_max$tmaxC))
qqnorm(log(t_max$tmaxC))
qqline(log(t_max$tmaxC))

# Coho/summer chinook/fall chinook peak flow
# Data have long tail
par(mfrow=c(2,2))
plot(density(sno_q$peak_q_coho))
hist(sno_q$peak_q_coho)
qqnorm(sno_q$peak_q_coho)
qqline(sno_q$peak_q_coho)
# Log transformation
par(mfrow=c(2,2))
plot(density(log(sno_q$peak_q_coho)))
hist(log(sno_q$peak_q_coho))
qqnorm(log(sno_q$peak_q_coho))
qqline(log(sno_q$peak_q_coho))

# Winter/summer steelhead peak flow
# Data have long tail
par(mfrow=c(2,2))
plot(density(sno_q$peak_q_winter_steelhead))
hist(sno_q$peak_q_winter_steelhead)
qqnorm(sno_q$peak_q_winter_steelhead)
qqline(sno_q$peak_q_winter_steelhead)
# Log transformation
par(mfrow=c(2,2))
plot(density(log(sno_q$peak_q_winter_steelhead)))
hist(log(sno_q$peak_q_winter_steelhead))
qqnorm(log(sno_q$peak_q_winter_steelhead))
qqline(log(sno_q$peak_q_winter_steelhead))

# Low flow
par(mfrow=c(2,2))
plot(density(sno_q$low_q7))
hist(sno_q$low_q7)
qqnorm(sno_q$low_q7)
qqline(sno_q$low_q7)
# Log transformation
par(mfrow=c(2,2))
plot(density(log(sno_q$low_q7)))
hist(log(sno_q$low_q7))
qqnorm(log(sno_q$low_q7))
qqline(log(sno_q$low_q7))


# Prepare data for MARSS model
sno <- sno_q
sno[,c(2)] <- log(sno[,c(2)]) #log transform coho (representing coho and both chinook runs)
sno[,c(5)] <- log(sno[,c(5)]) #log transform winter sthd (representing both steelhead runs)
sno[,c(7)] <- log(sno[,c(7)])
sno <- scale(sno[,c(2,5,7)])
sno_mean <- attributes(sno)$'scaled:center'
sno_std <- attributes(sno)$'scaled:scale'
sno <- t(sno)
air_t <- t_max
air_t[,c(2)]  <- log(air_t[,c(2)])
air_t <- scale(air_t[, 2])
air_t_mean <- attributes(air_t)$'scaled:center'
air_t_std <- attributes(air_t)$'scaled:scale'
air_t <- t(air_t)
dat_full <- rbind(sno, air_t)
rownames(dat_full)[4] <- 'air_t'
colnames(dat_full) <- 1963:2021
numYears<-ncol(dat_full)

#fitting MARSS model
dat <- dat_full
Vars <- row.names(dat)
numVars <- length(Vars)
years <- 1963:2021

# MARSS fit
mod.list.noint <- list(
  B="diagonal and unequal",
  U="zero",
  Q="unconstrained",
  Z="identity",
  A="zero",
  R="zero",
  x0="zero",
  tinitx=0 )

ts.ch.1 <- MARSS(dat,
                 model = mod.list.noint,
                 fun.kf ="MARSSkfss",
                 control=list(maxit=5000,
                              conv.test.slope.tol=0.001,
                              trace=1,
                              safe=TRUE))



# MARSS model residuals (innovations) with intervals
d <- residuals(ts.ch.1, type="tt1")
d$.conf.low <- d$.fitted+qnorm(0.05/2)*d$.sigma
d$.conf.up <- d$.fitted-qnorm(0.05/2)*d$.sigma
ggplot(data = d) +
  geom_line(aes(t, .fitted)) +
  geom_point(aes(t, value), na.rm=TRUE) +
  geom_ribbon(aes(x = t, ymin = .conf.low, ymax = .conf.up), linetype = 2, alpha = 0.1) +
  ggtitle("Model residuals (innovations)") +
  xlab("Time Step") + ylab("Count")

# Time series of the states
colr<-rainbow(numVars)
op<-par(mfrow=c(numVars,1), mar=c(4,4,0.1,0), oma=c(0,0,2,0.5))
for(i in 1:numVars) {
  mn = ts.ch.1$states[i,]
  se = ts.ch.1$states.se[i,]
  plot(years, mn, xlab="", ylab=Vars[i], bty="n", xaxt="n", type="n",
       ylim=c(min(mn-2*se, dat[i,], na.rm=TRUE), max(mn+2*se, dat[i,], na.rm=TRUE)))
  lines(years, rep(0,numYears), lty="dashed")
  lines(years, mn, col=colr[i], lwd=3)
  lines(years, mn+2*se, col=colr[i])
  lines(years, mn-2*se, col=colr[i])
  lines(years, dat[i,], pch=16,
        #type='b',
        col="black")
}
axis(1,at=seq(min(years),max(years)))
par(op)



# MARSS fitted model variance/covariance matrix
coef(ts.ch.1)$Q
Qmat <- matrix(0, numVars, numVars)
Qmat[lower.tri(Qmat, diag=TRUE)]<-ts.ch.1$par$Q
rownames(Qmat)<-Vars
colnames(Qmat)<-Vars
# Color coding scheme
rgb.palette.pos <- colorRampPalette(c("red", "white"), space = "rgb")
rgb.palette.neg <- colorRampPalette(c("white", "blue"), space = "rgb")
colours <- c("#000000",rgb.palette.pos(120), rep("#FFFFFF", 60), rgb.palette.neg(120), "#000000")
# Correlation matrix plot
lattice::levelplot(Qmat, scales=list(x=list(rot=90)),
                   col.regions=colours, cuts=300, at=seq(-1,1,0.01), xlab="", ylab="")
# MARSS covariance matrix
Qmat


# Chinook and coho peak flow data and simulations
plot(1:59, dat[1,], type="l", col=NA, main="Snohomish peak flow coho & chinook",
     ylim=c(-4,4), xlim=c(1,105))
for(i in 1:numSims){
  lines(1:105, mySims$sim.dat[1,1:105,i], col="orangered")
}
lines(1:59, dat[1,], col="black", lwd=2)

# Data simulation
simYears<-150
numSims<-100
mySims <- MARSSsimulate(ts.ch.1, tSteps = simYears, nsim = numSims)

# Example simulation from fitted MARSS model
# row 1 = coho/chinook, col 1 = 1/100 simulations
plot(1:simYears, mySims$sim.data[1,,1], type="l", col="orangered",
     ylab="Simulated response",
     ylim=c(-4,4))
lines(1:simYears, mySims$sim.data[2,,1], col="forestgreen")
lines(1:simYears, mySims$sim.data[3,,1], col="blue")
lines(1:simYears, mySims$sim.data[4,,1], col="purple")
legend("bottomleft", legend=c("peak.q.coho.chin","peak.q.win.sum.sthd",
                              "low.q","airTMAX" ),
       lty=1, lwd=4,
       ncol=3, bty="n",
       col=c("orangered", "forestgreen","blue", "purple"))

# Diagnostic plots
# All 100 simulations (colored lines) and transformed and scaled data (black line)
# Low flow data and simulations
par(mfrow=c(1,1))
par(bty="l")
par(mar=c(5,5,1,1))
plot(1:59, dat[3,], type="l", col=NA, main="Snohomish low flow",
     ylim=c(-4,4), xlim=c(1,105), ylab= "log(low flow)", xlab="Simulation year")
for(i in 1:numSims){
  lines(1:105, mySims$sim.dat[3,1:105,i], col="blue")
}
lines(1:59, dat[3,], col="black", lwd=2)

# Chinook and coho peak flow data and simulations
plot(1:59, dat[1,], type="l", col=NA, main="Snohomish peak flow coho & chinook",
     ylim=c(-4,4), xlim=c(1,105))
for(i in 1:numSims){
  lines(1:105, mySims$sim.dat[1,1:105,i], col="orangered")
}
lines(1:59, dat[1,], col="black", lwd=2)

# Winter and summer steelhead peak flow data and simulations
plot(1:59, dat[2,], type="l", col=NA, main="Snohomish peak flow summer and winter steelhead",
     ylim=c(-4,4), xlim=c(1,105))
for(i in 1:numSims){
  lines(1:105, mySims$sim.dat[2,1:105,i], col="forestgreen")
}
lines(1:59, dat[2,], col="black", lwd=2)

#Max air temperature data and simulations
plot(1:59, dat[4,], type="l", col=NA, main="Stillaguamish air TMAX",
     ylim=c(-4,4), xlim=c(1,105))
for(i in 1:numSims){
  lines(1:105, mySims$sim.dat[4,1:105,i], col="purple")
}
lines(1:59, dat[4,], col="black", lwd=2)


# Diagnostic density plots with simulated data and transformed and z-scored data
# Transformed/z-scored flow or temp data (black line)
# Simulated data for all 100 years (colored lines)
png(filename="snoh_transformed_diagnostic_density.png", res=200, height=5, width=5, units="in")
par(mfrow=c(3,2))
par(mar=c(3,2,2,2))
par(oma=c(2,1,1,1))
# Low flow
plot(density(dat[3,]), col="black", lwd=3, main="Low flow",
     ylim=c(0,0.6))
for(i in 1:numSims){
  lines(density(mySims$sim.data[3,,i]), col="lightblue")
}
lines(density(dat[3,]), col="black", lwd=3)
# Peak flow coho and chinook
plot(density(dat[1,]), col="black", lwd=3, main="Peak flow coho and chinook",
     ylim=c(0,0.6))
for(i in 1:numSims){
  lines(density(mySims$sim.data[1,,i]), col="orangered")
}
lines(density(dat[1,]), col="black", lwd=3)
# Peak flow winter and summer steelhead
plot(density(dat[2,]), col="black", lwd=3, main="Peak flow steelhead",
     ylim=c(0,0.6))
for(i in 1:numSims){
  lines(density(mySims$sim.data[2,,i]), col="forestgreen")
}
lines(density(dat[2,]), col="black", lwd=3)
# Max air temperature
plot(density(dat[4,]), col="black", lwd=3, main="Air TMAX",
     ylim=c(0,0.6))
for(i in 1:numSims){
  lines(density(mySims$sim.data[4,,i]), col="purple")
}
lines(density(dat[4,]), col="black", lwd=3)
dev.off()

# Back-transform data
# https://stackoverflow.com/questions/46397553/r-transform-scaled-and-centered-data-to-original-values
# value * 'scaled:scale' + 'scaled:center'
# = value * std + mean
# back-transform lowQ, peakQ, airTMAX
test <- apply(mySims$sim.data, MARGIN=c(2,3), function(x) x * t(c(sno_std, air_t_std)) +
                t(c(sno_mean, air_t_mean)))
test2 <- test
#test2[,,1] lets you see the entire back transformed simulated data for 1 simulation

# one more step:
# inverse log transform of low flow and max air temperature
# inverse square root transform (i.e., square) peak flow summer steelhead
for(i in 1:dim(test2)[3]){ #why the [3]?
  test2[1,,i] <- exp(test2[1,,i])
  test2[2,,i] <- exp(test2[2,,i]) #(test2[2,,i]^2)
  test2[3,,i] <- exp(test2[3,,i])
  test2[4,,i] <- exp(test2[4,,i])
}

# Diagnostic plots with back-transformed data and back-transformed model simulations
png(filename="snoh_backtransformed_diagnostic_density.png", res=200, height=5, width=5, units="in")
par(mfrow=c(3,2))
par(mar=c(3,2,2,2))
par(oma=c(2,1,1,1))
# Low flow
plot(density(exp(dat[3,]*sno_std[3] + sno_mean[3])),
     col="black", lwd=3,
     main="Low flow",
    ylim=c(0,0.00125))
for(i in 1:numSims){
  lines(density(test2[3,,i]), col="lightblue")
}
lines(density(exp(dat[3,]*sno_std[3] + sno_mean[3])), col="black", lwd=3)
# Peak flow coho and chinook
plot(density(exp(dat[1,]*sno_std[1] + sno_mean[1])),
     col="black", lwd=3,
     main="Peak flow coho and chinook",
     ylim=c(0,0.00004))
for(i in 1:numSims){
  lines(density(test2[1,,i]), col="orangered")
}
lines(density(exp(dat[1,]*sno_std[1] + sno_mean[1])), col="black", lwd=3)
# Peak flow winter steelhead
plot(density(exp(dat[2,]*sno_std[2] + sno_mean[2])),
     col="black", lwd=3,
     main="Peak flow steelhead",
     ylim=c(0,0.00003))
for(i in 1:numSims){ #this line still wont work
  lines(density(test2[2,,i]), col="forestgreen")
}
lines(density(exp(dat[2,]*sno_std[2] + sno_mean[2])), col="black", lwd=3)
# Max air temperature
plot(density(exp(dat[4,]*air_t_std + air_t_mean)),
     col="black", lwd=3,
     main="Air TMAX",
     ylim=c(0,0.25))
for(i in 1:numSims){
  lines(density(test2[4,,i]), col="purple")
}
lines(density(exp(dat[4,]*air_t_std + air_t_mean)), col="black", lwd=3)
dev.off()

# Back-trasnformed histogram plots
png(filename="snoh_coho_chin_histogram.png", res=200, height=5, width=5, units="in")
par(mfrow=c(2,1))
hist(test2[1,,], main="MARSS simulations of peak flow (coho/chinook)",
     xlim=c(0,140000))
hist(exp(dat[1,]*sno_std[1] + sno_mean[1]),
     xlim=c(0,140000),
     main= "Observed peak flow at Snohomish near Monroe (coho/chinook)")
dev.off()

png(filename="snoh_sthd_histogram.png", res=200, height=5, width=5, units="in")
par(mfrow=c(2,1))
hist(test2[2,,], main="MARSS simulations of peak flow (steelhead)", xlim=c(0,150000))
hist(exp(dat[2,]*sno_std[2] + sno_mean[2]),xlim=c(0,150000),
     main= "Observed peak flow at Snohomish near Monroe (steelhead)")
dev.off()

png(filename="snoh_low_flow_histogram.png", res=200, height=5, width=5, units="in")
par(mfrow=c(2,1))
hist(test2[3,,],main="MARSS simulations of low flow", xlim=c(0,3500), freq=FALSE)
density_test<-density(test2[3,,])
lines(density_test, lwd=2, col="red")
hist(exp(dat[3,]*sno_std[3] + sno_mean[3]), xlim=c(0,3500),
     main= "Observed low flow at Snohomish near Monroe", freq=FALSE)
density_two<-density(exp(dat[3,]*sno_std[3] + sno_mean[3]))
lines(density_two, lwd=2, col="red")
dev.off()

png(filename="snoh_tmax_air_histogram.png", res=200, height=5, width=5, units="in")
par(mfrow=c(2,1))
hist(test2[4,,], main="MARSS simulations of TMAX air", xlim=c(25,45))
hist(exp(dat[4,]*air_t_std + air_t_mean), xlim=c(25,45),
     main= "Observed TMAX air at Monroe")
dev.off()
