# Snohomish version of Jeff Jorgensen's Chehalis Time Series RMD
# To simulate time series while preserving correlations and covariances between them, we can use MARSS models
# The following represent Stillaguamish peak flow, low flow, and air temperature data time series to forward simulate

require(MARSS)
library(tidyverse)

# The peak flow data will be summarized for the incubation period for each population.
incubation_range <- list(
  c(pop = 'coho',           start = "1015", end = "0515"),
  c(pop = 'summer_chinook', start = "0910", end = "0415"),
  c(pop = 'fall_chinook',   start = "0920", end = "0510"),
  c(pop = 'winter_steelhead', start = "0301", end = "0910"),
  c(pop= 'summer_steelhead', start = "0301", end = "0910"))

# Daily discharge at Snohomish near Monroe
require(EGRET)
site_number_sno <- 12150800
start_date <- "1963-02-01"  # Get earliest date
end_date <- "2020-12-31"
daily_sno <- readNWISDaily(site_number_sno,"00060",start_date,end_date)

# Find the annual maximum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
sno_q_peak <- map_dfr(incubation_range, function(r){
  daily_sno %>%
    mutate(monthday = format(Date, "%m%d"),
           pop = r['pop']) %>%
    filter(monthday <= r['start'] | monthday >= r['end']) %>%
    group_by(pop, waterYear) %>%
    summarize(Q_max = max(Q) * 35.31467)
})
sno_q <- pivot_wider(sno_q_peak,
                     names_from = pop,
                     names_prefix = 'peak_q_',
                     values_from = Q_max)
# Find the annual minimum peak flow during the incubation period for each pop
# Subset data to species specific range, return annual max
# Convert from cubic meters per second to cubic feet per second
# Calculating 7DADMin for low flow
sno_low_q7 <- tapply(daily_sno$Q7[daily_sno$waterYear>1962],
                     daily_sno$waterYear[daily_sno$waterYear>1962],
                     function(x, n = 7) {
                       min(as.vector(stats::filter(x, rep(1/n, n),sides=2)),
                           na.rm = TRUE)
                     })
sno_low_q7 <- data.frame(waterYear = names(sno_low_q7),
                         low_q7 = sno_low_q7 * 35.31467)

sno_q <- merge(sno_q, sno_low_q7)

# Stream temperature data from NorWest lower Snoqualmie
# Used to create relationship between max air temp and max water temp
snoq_stream_temp<-read.csv("data-raw/data_stochastic/Snoq_lower_stream_temp.csv")%>%
  mutate(start_date = as.Date(start_date),
         end_date = as.Date(end_date))

# Air temperature data
# USC00455525 is the ID for the Snohomish Monroe NOAA NCDC station
require(rnoaa)
require(purrr)

t_max_yr <- map(c("USC00455525"), ghcnd_search,
             var = 'TMAX',
             date_min = '1920-01-01',
             date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y")))%>% #year only format for year_1 column
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)

t_max <- t_max_yr[35:93,] #1963 - 2021
# Subset air temperature time period to match stream temperature time series
t_max_regression <- map(c("USC00455525"), ghcnd_search,
                        var = 'TMAX',
                        date_min = '1920-01-01',
                        date_max = '2021-12-31') %>%

  map_dfr("tmax")%>%
  mutate(year = as.numeric(format(date, "%Y"))) #year only format for year_1 column

date_subset <- function(start_date, end_date){
  filter(t_max_regression, date>=start_date, date<=end_date)

}

t_max_subset <- map2_dfr(snoq_stream_temp$start_date, snoq_stream_temp$end_date, date_subset)%>%
  group_by(year)%>%
  summarise(tmaxC = max(tmax, na.rm=TRUE)*0.10)

# We fit a relationship between Snohomish TMAX air temperature and NorWest  Snoqualmie 7DADM water temperature
# This relationship is used later to convert the MARSS simulation of air TMAX to water 7DADM
# Subset air temperature time period to match stream temperature time series
stream_air_temp <- merge(x=snoq_stream_temp,
                         y=t_max_subset, all.x = TRUE)

# Regression
lm_7dadm_tmax <- lm(T7dadm ~ tmaxC, stream_air_temp)
summary(lm_7dadm_tmax)

# Prepare data for MARSS model
# Before bringing data into a MARSS model, transform data that is not normally distributed
# Here, we log transform the low flow, peak flow for all spp, and tmax data
# Then center and scale (z-score) the data
sno <- sno_q
sno[,c(2)] <- log(sno[,c(2)]) #log transform coho (representing coho and both chinook runs)
sno[,c(5)] <- log(sno[,c(5)]) #log transform winter sthd (representing both steelhead runs)
sno[,c(7)] <- log(sno[,c(7)]) #log transform low flow

# Coho and chinook have a 1:1 match, so drop chinook columns
sno <- scale(sno[,c(2,5,7)])

# Column means & standard deviations needed later for back-transforming
sno_mean <- attributes(sno)$'scaled:center'
sno_std <- attributes(sno)$'scaled:scale'
sno <- t(sno)

# Transform and z-score TMAX air temperature
air_t <- t_max
air_t[,c(2)]  <- log(air_t[,c(2)])
air_t <- scale(air_t[, 2])

# Column means & stdevs needed later for back-transforming
air_t_mean <- attributes(air_t)$'scaled:center'
air_t_std <- attributes(air_t)$'scaled:scale'
air_t <- t(air_t)

# Combining data
dat_full <- rbind(sno, air_t)
rownames(dat_full)[4] <- 'air_t'
colnames(dat_full) <- 1963:2021
numYears<-ncol(dat_full)

# Fitting MARSS model
dat <- dat_full
Vars <- row.names(dat)
numVars <- length(Vars)
years <- 1963:2021

# MARSS fit
mod.list.noint <- list(
  B="diagonal and unequal", #why is this mean reverting instead of a random walk (diagonal and equal)
  U="zero",
  Q="unconstrained", #most parameterized option for estimated error correlatrion structure
  Z="identity", #grouping hypothesis matrix
  A="zero",
  R="zero", #error structure
  x0="zero",
  tinitx=0 )


ts.ch.1 <- MARSS(dat,
                 model = mod.list.noint,
                 fun.kf ="MARSSkfss",
                 control=list(maxit=5000,
                              conv.test.slope.tol=0.001,
                              trace=1,
                              safe=TRUE), silent = TRUE)
# Data simulation
simYears<-150
numSims<-100
mySims <- MARSSsimulate(ts.ch.1, tSteps = simYears, nsim = numSims)

# Back-transforming data
# https://stackoverflow.com/questions/46397553/r-transform-scaled-and-centered-data-to-original-values
# value * 'scaled:scale' + 'scaled:center'
# = value * std + mean
# back-transform lowQ, peakQ, airTMAX
test <- apply(mySims$sim.data, MARGIN=c(2,3), function(x) x * t(c(sno_std, air_t_std)) +
                t(c(sno_mean, air_t_mean)))
test2 <- test

# one more step:
# inverse log transform of low flow and max air temperature
# inverse square root transform (i.e., square) peak flow summer steelhead
for(i in 1:dim(test2)[3]){ #why the [3]?
  test2[1,,i] <- exp(test2[1,,i])
  test2[2,,i] <- exp(test2[2,,i])
  test2[3,,i] <- exp(test2[3,,i])
  test2[4,,i] <- exp(test2[4,,i])
}



# Add data to `harp` package

#Store the unscaled version of the three MARSS forecasts. This makes them available for other functions to call.
dat_marss <- test2

dimnames(dat_marss) <- list(#var = rownames(mySims$MLEobj$model$data),
  var = c('q_peak', 'q_peak_sthd', 'q_low', 't_peak'),
  year = 1:simYears,
  run = 1:numSims)


# Convert array to dataframe
df_marss <- as.data.frame.table(dat_marss) %>%
  rename(value = Freq)


